<?php

require_once("Types.php");

function getCmdOptions($argc = null, $argv = null)
{
	return CmdOptions::getInstance($argc, $argv);
}

class CmdOptions
{
	const OS   = 'os';
	const PATH = 'path';
	const WIN  = 'win';
	const CFG_FILE = 'cfg-file';
	const RELEASE  = 'release';
	const SRC_HOST = 'src-host';
	const PRIVATE_KEY = 'private-key';
	const PUBLIC_KEY  = 'public-key';

	private static $instance;
	private $options;

	public static function getInstance($argc = null, $argv = null)
	{
		if (!self::$instance) {
			if (!$argc || !$argv)
				throw new Mirror_Exception("argc/argv are not defined for CmdOptions class instance");
			self::$instance = new CmdOptions($argc, $argv);
		}
		return self::$instance;
	}

	function __construct($argc, $argv)
	{
		$this->get_options($argc, $argv);
		return $this;
	}

	private function get_options($argc, $argv)
	{
		global $options;

		if ($argc === 1) {
			$this->options[Types::CMD] = Types::HELP;
			return;
		}

		$optslist = array(
				// Commands
				"show-all-releases" => Types::CMD,
				"product-list" => Types::CMD,
				"os-list"  => Types::CMD,
				"cfg"      => Types::CMD,
				"update"   => Types::CMD,
				"push"     => Types::CMD,
				"destroy"  => Types::CMD,
				"version"  => Types::CMD,
				"help"     => Types::CMD,

				// Flags and additional commands
				"add"      => Types::FLAG,
				"remove"   => Types::FLAG,
				"import"   => Types::FLAG,
				"export"   => Types::FLAG,
				"check"    => Types::FLAG,
				"agree"    => Types::FLAG,
				"win"      => Types::FLAG,

				// Arguments
				"release"  => Types::ARG,
				"src-host" => Types::ARG,
				"path"     => Types::ARG,
				"os"       => Types::ARG,
				"cfg-file" => Types::ARG,
				"private-key" => Types::ARG,
				"public-key"  => Types::ARG
		);

		for ($i = 1; $i <= $argc; $i++) {
			// Get command
			if ($i == 1) {
				if (isset($optslist[$argv[$i]]) && ($optslist[$argv[$i]] === Types::CMD)) {
					// Actually it is a hidden conversion
					// from string to Types type
					$this->options[Types::CMD] = $argv[$i];
				} else {
					throw new Mirror_Cmd_Exception("Unknown command: " . $argv[$i]);
				}
				continue;
			}
			// Get flags and options
			if (preg_match("/^--([a-zA-Z0-9-]+)/", $argv[$i], $match)) {
				if (isset($optslist[$match[1]])) {
					if ($optslist[$match[1]] == Types::FLAG)
						$this->options[$match[1]] = true;

					if ($optslist[$match[1]] == Types::ARG) {
						// FIXME: Add strong command line validation
						if (!preg_match("/^--([a-zA-Z0-9-]+)=(.*)$/", $argv[$i], $match))
							throw new Mirror_Cmd_Exception("Option has been defined wrong: " . $argv[$i]);
						$this->options[$match[1]] = $match[2];
					}
				}
			}
		}

		if ($this->options[self::WIN]) {
			if ($this->options[self::OS] || $this->options[Types::OS_LST])
				throw new Mirror_Cmd_Exception("'--os' and 'os-list' are not supported for Windows repository");
			$this->options[self::OS] = "all";
		}

		// Split path variable by parameters(login, passwd, host, port, path)
		if ($this->options[self::PATH]) {
			$tmp = $this->parse_path($this->options[self::PATH]);
			$this->options[self::PATH] = $tmp;
		}

		// repository url
		if ($this->options[self::SRC_HOST]) {
			$options['repos'] = $this->options[self::SRC_HOST];
		} else {
			$options['repos']  = $options['repos_rsync'];
			if ($this->options[self::WIN])
				$options['repos']  = $options['repos_rsync_win'];
		}
	}

	public function is_set($flag)
	{
		return ((bool)$this->options[$flag]);
	}

	public function is_win()
	{
		return ((bool)$this->options[self::WIN]);
	}

	public function get_cmd()
	{
		return $this->options[Types::CMD];
	}

	public function get_release()
	{
		return $this->options[self::RELEASE];
	}

	public function get_os()
	{
		return $this->options[self::OS];
	}

	public function get_path_opts()
	{
		return $this->options[self::PATH];
	}

	public function get_src_host()
	{
		return $this->options[self::SRC_HOST];
	}

	public function get_path()
	{
		return $this->options[self::PATH]['path'];
	}

	public function get_cfg_file()
	{
		return $this->options[self::CFG_FILE];
	}

	public function set($opt, $val)
	{
		$this->options[$opt] = $val;
	}

	private function parse_path($path)
	{
		if (!isset($path) || empty($path))
			throw new Mirror_Cmd_Exception("Wrong credentials/path have been defined");

		$data = array();

		// Split parameters by '@' symbol
		$res = preg_match("/^(.*)@(.*)$/", $path, $matches);

		// parse host:[port]/path/to/repository
		$data = $this->parse_path_opts($res ? $matches[2] : $path , $data);

		// Is local path
		if (!$data['host'])
			return $data;

		// login[:passwd] has been defined
		$data['login'] = $_SERVER['USER'];
		if ($res) {
			$data = $this->get_login($matches[1], $data);
		}

		// Trying to find ssh session keys
		$data = $this->find_ssh_keys($data);

		return $data;
	}

	private function parse_path_opts($str, $data)
	{
		$res = preg_match("/^(.*):(\d+)?(.*)/", $str, $opts);

		// Is local path
		if (!$res) {
			if (!$str)
				throw new Mirror_Cmd_Exception("Please specify the destination path at first");
			$data['path'] = $str;
			return $data;
		}

		// Getting host:port/path
		$data['host'] = $opts[1];
		$data['port'] = $opts[2] ? $opts[2] : 22;
                $data['path'] = $opts[3];

		return $data;
	}

	private function get_login($str, $data)
	{
		$res = preg_match("/^(.*):(.*)$/", $str, $opts);
		if (!$res && !$str)
			throw new Mirror_Cmd_Exception("login is not defined");

		$data['login']  = $opts[1] ? $opts[1] : $str;
		$data['passwd'] = $opts[2] ? $opts[2] : "";

		return $data;
	}

	private function find_ssh_keys($data)
	{
		// get home directory. A little hack to do not use posix_getpwnam
		// because of an additional requirement will appeared.
		$content = file("/etc/passwd", FILE_SKIP_EMPTY_LINES);
		if (!$content)
			throw new Mirror_Exception("Unable to read passwd file");

		$base_path = '';
		foreach ($content AS $str) {
			list($user, $pass, $uid, $gid, $gecos, $home, $shell) = explode(":", $str);
			if ($data['login'] == $user) {
				$base_path = $home;
				break;
			}
		}
		if ($base_path == "")
			return $data;

		$base_path .= '/.ssh';

		if ($this->options[self::PRIVATE_KEY] || $this->options[self::PUBLIC_KEY]) {
			if (!$this->options[self::PRIVATE_KEY] || !$this->options[self::PUBLIC_KEY])
				throw new Mirror_Cmd_Exception("private or public keys are note defined");

			$data['ssh_priv_key']   = $this->options[self::PRIVATE_KEY];
			$data['ssh_pub_key'] = $this->options[self::PUBLIC_KEY];

			return $data;
		}

		if (file_exists($base_path . '/id_rsa.pub')) {
			$data['ssh_pub_key'] = $base_path . '/id_rsa.pub';
		} else if (file_exists($base_path . '/id_dsa.pub')) {
			$data['ssh_pub_key'] = $base_path . '/id_dsa.pub';
		}
		$data['ssh_priv_key'] = str_replace('.pub', '', $data['ssh_pub_key']);

		return $data;
	}
}
