<?php

require_once("Product.php");

class Config
{
	private $filename_default = "config.xml";
	private $filename;
	private $data; // raw simplexml data

	function __construct($file = null)
	{
		global $options;

		if (substr($file, -1) == '/')
			throw new Mirror_Config_Exception("Wrong config file name: $file");

		$file ? $file : ($file = $options['tmpdir'] . "/" . $this->filename_default);
		$this->filename = $file;

		// Parse config file
		$this->pull();

		return $this;
	}

	private function pull()
	{
		echo_try("read config file: " . $this->filename);
		if (!file_exists($this->filename)) {
			$xml = '<?xml version="1.0" encoding="UTF-8" ?><xml/>';
			$this->data = simplexml_load_string($xml);
			echo_ok();
			return;
		}

		$this->data = simplexml_load_file($this->filename);
		if (!$this->data) {
			throw new Mirror_Exception("Unable to load the config file '"
				. $this->filename . "'");
		}
		echo_ok();
	}

	private function push()
	{
		save_config($this->filename, $this->data);
	}

	private function read_release_version($release)
	{
		global $options;

		$product = Product::factory(Types::PLESK);
		$data = $product->get_data();
		$version = $data[$release]['info']['version'];
		return $version;
	}

	// FIXME: possibly deprecated
	public function get_release_list()
	{
		$list = array();
		foreach($this->data->release as $entry)
			$list[] = (string)$entry->attributes()->name;

		return $list;
	}

	// FIXME: possibly deprecated
	public function get_os_list($release)
	{
		$list = array();

		$oses = $this->data->xpath('//release[@name="'.$release.'"]/os');
		foreach ($oses as $os)
			$list[] = (string)$os->attributes()->name;

		return $list;
	}

	public function get_release_version($release)
	{
		if (!$this->data->release)
			return;

		foreach ($this->data->release AS $rel) {
			if ($rel->attributes()->name == $release)
				return (string)$rel->attributes()->version;
		}

		return;
	}

	public function add($release, $os)
	{
		global $options;

		if (!isset($release) || !isset($os) || empty($release) || empty($os))
			throw new Mirror_Cmd_Exception("Some parameters were not defined");

		$cfg_release = '';
		if ($this->data->release) {
			$item = $this->data->xpath('//release[@name="'.$release.'"]');
			$cfg_release = $item[0];
		}

		if (!$cfg_release) {
			$cfg_release = $this->data->addChild('release');
			$cfg_release->addAttribute('name', $release);
			$cfg_release->addAttribute('version', $this->read_release_version($release));
		}

		$product = Product::factory(Types::PLESK);
		$oslist = $product->get_os_list($release);
		foreach ($oslist AS $key) {
			if ($os !== "all" && $os != $key)
				continue;

			echo_try("add OS '$key' into config file");
			if (!$this->os_exists($cfg_release, $key)) {
				$child = $cfg_release->addChild('os');
				$child->addAttribute('name', $key);
				echo_ok();
			} else {
				echo_skip("already exists");
			}
		}
		$this->push();
	}

	private function os_exists($elem, $os)
	{
		if (!$elem->os)
			return false;

		$key = $elem->xpath('os[@name="'.$os.'"]');
		if (count($key))
			return true;

		return false;
	}

	public function remove($release, $os)
	{
		echo_try("remove OS '$os' from config file");
		if (!isset($release) || !isset($os) || empty($release) || empty($os))
			throw new Mirror_Cmd_Exception("Some parameters are not defined");

		// Already deleted
		if (!$this->data->release) {
			echo_skip("not exists");
			return;
		}

		$item = $this->data->xpath('//release[@name="'.$release.'"]');
		$cfg_release = $item[0];
		foreach($cfg_release->os as $entry) {
			if ($entry->attributes()->name == $os) {
				$domRef = dom_import_simplexml($entry);
				$domRef->parentNode->removeChild($domRef);
			}
		}

		// Remove empty release section
		if (!$cfg_release->os || $os == "all") {
			$domRef = dom_import_simplexml($cfg_release);
			$domRef->parentNode->removeChild($domRef);
		}
		echo_ok();
		$this->push();
	}

	public function check()
	{
		if (!file_exists($this->filename))
			throw new Mirror_Config_Exception("Config file: " . $this->filename . " doesn't found");

		$product = Product::factory(Types::PLESK);
		$releases = $product->get_release_list();
		$opts = getCmdOptions();

		foreach($this->data->releases as $cfg_release) {
                        $winflag = preg_match("/^(.*)_WIN$/", $cfg_release);
                        if (($winflag && !$opts->is_win()) || (!$winflag && $opts->is_win()))
                                throw new Mirror_Config_Exception("Mixed configuration (Windows/Unix) has been used");

			$my_release = (string)$cfg_release->attributes()->name;
			if (!in_array($my_release, $releases, true))
				throw new Mirror_Config_Exception("Unknown release '" . $my_release . "' has been defined in config.");

			if (!$cfg_release->os)
				return;

			$oses = $product->get_os_list($my_release);
			foreach ($cfg_release->os AS $my_os) {
				if (!in_array((string)$my_os->attributes()->name, $oses, true))
					throw new Mirror_Config_Exception("Unknown OS '" . $my_os->attributes()->name . "' has been defined in config for release " . $my_release);
			}
		}
	}

	public function export()
	{
		$opts = getCmdOptions();
		$action = Action::factory();
		$action->upload($this->filename, $opts->get_path() . "/mirror.xml" ,0644);
	}

	public function import()
	{
		$opts = getCmdOptions();
		$action = Action::factory();
		$action->download($opts->get_path() . "/mirror.xml", $this->filename ,0644);
	}

}
